import os
import argparse
from tqdm import tqdm
import torch
import torch.nn as nn
import numpy as np
from PIL import Image

import eqben.eqben_utils as eqben_utils
import eqben.eqben_acc as eqben_acc


##################### 0. IMPORT USER-MODEL PACKAGE ###########################
# In this template, we use "USER_MODEL" to represent the VLP model which would be evaluated.
# Please check other files within the example folder for the specific VLP model.
import clip



##################### 1. DATA SET PATH ###########################
# Please identify the path of "img_root" and "ann_root" in the server.

DATASET_INFO = {
    "eqben": {"func": eqben_utils.EqBenALL,
              "img_root": '/home/wangtan/code/ms_intern_github/remedy/private_eqben/source/image',
              "ann_root": '/home/wangtan/code/ms_intern_github/remedy/private_eqben/source/after_random/ann_json_finegrained_random.json'},
    'winoground': {"func": eqben_utils.Winoground,
                   'img_root':'/home1/wangtan/data/winoground/images',
                   'ann_root':'/home1/wangtan/data/winoground/examples.jsonl'},
    'valse': {"func": eqben_utils.VALSE,
              'img_root':'/home1/wangtan/data/valse/valse_ann/ours/img',
              'ann_root':'/home1/wangtan/data/valse/valse_ann/ours/global_ann.json'}
}



################ 2. CUSTOMIZED DATA TOOL #####################
# In USER_MODEL, how will the data (image/caption) be pre-processed?
# How will the data be collated in the batch?

class customized_data_toolkit():
    def __init__(self, transform):
        self.transform = transform

    def process_img_pixel(self, image):
        # Define the model-specifc data pre-process (e.g., augmentation) for image pixel.
        image = Image.open(image).convert("RGB")
        return self.transform(image)

    def process_img_npy(self, image):
        # Define the model-specifc data pre-process (e.g., augmentation) for image numpy file (Youcook2).
        image = Image.fromarray(np.load(image)[:, :, [2, 1, 0]], 'RGB')
        return self.transform(image)



################ 3. USER_MODEL INFERENCE PROCESS #####################
# For USER_MODEL, how to forward the data (image/caption) and obtain the image-text simiarity during the inference stage.

def main(config):

    # 3.1. SETUP LOGGER
    load_file_basename = 'clip_rn50'
    eval_results_path = 'EqBen_results/{}'.format(load_file_basename)
    os.makedirs(eval_results_path, exist_ok=True)

    from eqben.eqben_logger import LOGGER as logger_txt, add_log_to_file
    logger_path = "{}/eval_{}_{}_log.txt".format(eval_results_path, config.eval_task, config.eval_data)
    logger_txt.info("creating log at: {}".format(logger_path))
    add_log_to_file(logger_path)



    # 3.2. DATALOADER/USER_MODEL SETUP
    device = "cuda" if torch.cuda.is_available() else "cpu"
    model, preprocess = clip.load('RN50', device)
    model = model.to(device).eval()

    eval_dataset = DATASET_INFO[config.eval_data]["func"](img_root=DATASET_INFO[config.eval_data]["img_root"], ann_root=DATASET_INFO[config.eval_data]["ann_root"], config=config, customized_data_toolkit=customized_data_toolkit(preprocess))
    gpu_cnt = torch.cuda.device_count()
    print('use {} GPUs; '.format(gpu_cnt))
    eval_dataloader = torch.utils.data.DataLoader(eval_dataset, batch_size=int(gpu_cnt*128), num_workers=0, shuffle=False, collate_fn=eval_dataset.collate)
    clip_score_c0_i0_all, clip_score_c1_i0_all, clip_score_c0_i1_all, clip_score_c1_i1_all = [], [], [], []



    # 3.3. FORWARD PROCESS

    def clip_forward_batch_singlepair(model, image, caption):
        with torch.no_grad():
            text_encoded = model.encode_text(caption)
            text_encoded /= text_encoded.norm(dim=-1, keepdim=True)
            img_feature = model.encode_image(image)
            img_feature /= img_feature.norm(dim=-1, keepdim=True)

            sim = (text_encoded * img_feature).sum(dim=-1)
            return sim

    def clip_forward_batch(model, image0, image1, caption0, caption1):
        with torch.no_grad():
            text_encoded0 = model.encode_text(caption0)
            text_encoded0 /= text_encoded0.norm(dim=-1, keepdim=True)
            text_encoded1 = model.encode_text(caption1)
            text_encoded1 /= text_encoded1.norm(dim=-1, keepdim=True)

            img_feature0 = model.encode_image(image0)
            img_feature0 /= img_feature0.norm(dim=-1, keepdim=True)
            img_feature1 = model.encode_image(image1)
            img_feature1 /= img_feature1.norm(dim=-1, keepdim=True)

            sim_c0_i0 = (text_encoded0 * img_feature0).sum(dim=-1)
            sim_c0_i1 = (text_encoded0 * img_feature1).sum(dim=-1)
            sim_c1_i0 = (text_encoded1 * img_feature0).sum(dim=-1)
            sim_c1_i1 = (text_encoded1 * img_feature1).sum(dim=-1)

            return sim_c0_i0, sim_c1_i0, sim_c0_i1, sim_c1_i1


    if config.eval_data == "eqben":
        clip_score_all = []
        for idx, (image, caption) in enumerate(tqdm(eval_dataloader)):
            image = image.to(device)
            caption = clip.tokenize(caption).to(device)
            clip_score = clip_forward_batch_singlepair(model, image, caption)
            clip_score_all.append(clip_score.cpu())

        clip_score_all = torch.cat(clip_score_all, dim=0)
        np.save('{}/eqben_scores.npy'.format(eval_results_path), clip_score_all)


    else:
        for idx, (image0, image1, caption0, caption1) in enumerate(tqdm(eval_dataloader)):
            image0, image1 = image0.to(device), image1.to(device)
            caption0, caption1 = clip.tokenize(caption0).to(device), clip.tokenize(caption1).to(device)
            clip_score_c0_i0, clip_score_c1_i0, clip_score_c0_i1, clip_score_c1_i1 = clip_forward_batch(model, image0, image1, caption0, caption1)

            clip_score_c0_i0_all.append(clip_score_c0_i0.cpu())
            clip_score_c1_i0_all.append(clip_score_c1_i0.cpu())
            clip_score_c0_i1_all.append(clip_score_c0_i1.cpu())
            clip_score_c1_i1_all.append(clip_score_c1_i1.cpu())

        clip_score_c0_i0_all = torch.cat(clip_score_c0_i0_all, dim=0)
        clip_score_c1_i0_all = torch.cat(clip_score_c1_i0_all, dim=0)
        clip_score_c0_i1_all = torch.cat(clip_score_c0_i1_all, dim=0)
        clip_score_c1_i1_all = torch.cat(clip_score_c1_i1_all, dim=0)
        eval_clip_scores_itm = {'raw_info':eval_dataset.sample_pair, "c0_i0": clip_score_c0_i0_all, "c0_i1": clip_score_c0_i1_all, "c1_i0": clip_score_c1_i0_all, "c1_i1": clip_score_c1_i1_all}


        ################ 4. CALCULATE/SAVE THE PERFORMANCE #####################
        if 'valse' in config.eval_data:
            score_valse_acc, score_valse_min, score_valse_pair = eqben_acc.cal_valse_score(eval_clip_scores_itm)
            logger_txt.info("============ ITM Score =============")
            logger_txt.info("valse score pair: {}".format(score_valse_pair))
            logger_txt.info("valse score min(pc,pf): {}".format(score_valse_min))
            logger_txt.info("valse score acc: {}".format(score_valse_acc))

        else:
            text_score_itm, image_score_itm, group_score_itm = eqben_acc.cal_wino_score(eval_clip_scores_itm)
            eval_clip_scores_itm["scores"] = {"text_score":text_score_itm, "image_score":image_score_itm, "group_score":group_score_itm}
            logger_txt.info("============ ITM Score =============")
            logger_txt.info("text score: {}".format(text_score_itm))
            logger_txt.info("image score: {}".format(image_score_itm))
            logger_txt.info("group score: {}".format(group_score_itm))


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--eval_data', default='eqbenag')
    parser.add_argument('--eval_task', default='clip_pretrian')
    config = parser.parse_args()

    main(config)


import torch
import copy
import os
import json
from tqdm import tqdm
from torch.utils.data import Dataset
from PIL import Image
import numpy as np

import eqben.eqben_utils as eqben_utils
import eqben.eqben_acc as eqben_acc


##################### 0. IMPORT USER-MODEL PACKAGE ###########################
# In this template, we use "USER_MODEL" to represent the VLP model which would be evaluated.
# Please check other files within the example folder for the specific VLP model.
from fiber.config import ex
from fiber.modules import FIBERTransformerSS
from fiber.datamodules.datamodule_base import get_pretrained_tokenizer
from fiber.transforms import (
    albef_transform,
    albef_transform_randaug,
)

_transforms = {
    "albef": albef_transform,
    "albef_randaug": albef_transform_randaug,
}

##################### 1. DATA SET PATH ###########################
# Please identify the path of "img_root" and "ann_root" in the server.

DATASET_INFO = {
    "eqben": {"func": eqben_utils.EqBenALL,
              "img_root": '/home/wangtan/code/ms_intern_github/remedy/private_eqben/source/image',
              "ann_root": '/home/wangtan/code/ms_intern_github/remedy/private_eqben/source/after_random/ann_json_finegrained_random.json'},
    'winoground': {"func": eqben_utils.Winoground,
                   'img_root':'/home1/wangtan/data/winoground/images',
                   'ann_root':'/home1/wangtan/data/winoground/examples.jsonl'},
    'valse': {"func": eqben_utils.VALSE,
              'img_root':'/home1/wangtan/data/valse/valse_ann/ours/img',
              'ann_root':'/home1/wangtan/data/valse/valse_ann/ours/global_ann.json'}
}



################ 2. CUSTOMIZED DATA TOOL #####################
# In USER_MODEL, how will the data (image/caption) be pre-processed?
# How will the data be collated in the batch?

class customized_data_toolkit():
    def __init__(self, config):
        self.config = config
        self.tokenizer = get_pretrained_tokenizer(config["tokenizer"])

    def process_img_pixel(self, image):
        # Define the model-specifc data pre-process (e.g., augmentation) for image pixel.
        image = Image.open(image).convert("RGB")
        return _transforms[self.config["val_transform_keys"][0]](size=self.config["image_size"])(image)

    def process_img_npy(self, image):
        # Define the model-specifc data pre-process (e.g., augmentation) for image numpy file (Youcook2).
        image = Image.fromarray(np.load(image)[:, :, [2, 1, 0]], 'RGB')
        return _transforms[self.config["val_transform_keys"][0]](size=self.config["image_size"])(image)

    def get_img_batch(self, img_batch):
        batch_size = len(img_batch)
        # get max size
        img_sizes = list()
        img_sizes += [i.shape for i in img_batch]

        max_height, max_width = max([i[1] for i in img_sizes]), max([i[2] for i in img_sizes])
        new_img_batch = torch.zeros(batch_size, 3, max_height, max_width)
        for bi in range(batch_size):
            orig = img_batch[bi]
            new_img_batch[bi, :, :orig.shape[1], :orig.shape[2]] = orig
        return new_img_batch

    def collate(self, batch):
        # Define the model-specific data collate method, will be forwarded to "collate_fn"
        image0_batch, image1_batch = [item[0] for item in batch], [item[1] for item in batch]
        caption0_batch, caption1_batch = [item[2] for item in batch], [item[3] for item in batch]

        image0_batch_collate = self.get_img_batch(image0_batch)
        image1_batch_collate = self.get_img_batch(image1_batch)

        encode0_batch_collate = self.tokenizer(caption0_batch, padding="max_length",
                                              truncation=True, max_length=40, # config['max_text_len']
                                              return_tensors="pt",
                                              return_special_tokens_mask=True)

        encode1_batch_collate = self.tokenizer(caption1_batch, padding="max_length",
                                              truncation=True, max_length=40, # config['max_text_len']
                                              return_tensors="pt",
                                              return_special_tokens_mask=True)

        return image0_batch_collate, image1_batch_collate, encode0_batch_collate, encode1_batch_collate, caption0_batch, caption1_batch

    def collate_eqben(self, batch):
        # Define the model-specific data collate method, will be forwarded to "collate_fn"
        image_batch = [item[0] for item in batch]
        caption_batch = [item[1] for item in batch]

        image_batch_collate = self.get_img_batch(image_batch)

        encode_batch_collate = self.tokenizer(caption_batch, padding="max_length",
                                              truncation=True, max_length=40, # config['max_text_len']
                                              return_tensors="pt",
                                              return_special_tokens_mask=True)


        return image_batch_collate, encode_batch_collate, caption_batch



################ 3. USER_MODEL INFERENCE PROCESS #####################
# For USER_MODEL, how to forward the data (image/caption) and obtain the image-text simiarity during the inference stage.

def _loss_names(d):
    ret = {
        "itm": 0.5,
        "mlm": 0.,
        "itc":0.5,
        "vqa": 0,
        "nlvr2": 0,
        "caption_mle": 0,
        "caption_gold": 0,
        "caption_cider": 0,
    }
    ret.update(d)
    return ret


@ex.automain
def main(_config):
    _config = copy.deepcopy(_config)

    loss_names = _loss_names({"itm": 0.5})
    _config.update(
        {
            "loss_names": loss_names,
        }
    )

    # 3.1. SETUP LOGGER
    eval_results_path = os.path.basename(_config["load_path"])
    os.makedirs(os.path.join(_config["eval_root"], eval_results_path), exist_ok=True)

    from eqben.eqben_logger import LOGGER as logger_txt, add_log_to_file
    logger_path = "{}/{}/eval_{}_{}_log.txt".format(_config["eval_root"], eval_results_path, _config["eval_task"], _config["eval_data"])
    logger_txt.info("creating log at: {}".format(logger_path))
    add_log_to_file(logger_path)


    # 3.2. DATALOADER/USER_MODEL SETUP
    eval_dataset = DATASET_INFO[_config["eval_data"]]["func"](img_root=DATASET_INFO[_config["eval_data"]]["img_root"], ann_root=DATASET_INFO[_config["eval_data"]]["ann_root"], config=_config, customized_data_toolkit=customized_data_toolkit(_config))
    gpu_cnt = torch.cuda.device_count()
    print('use {} GPUs; '.format(gpu_cnt))
    eval_dataloader = torch.utils.data.DataLoader(eval_dataset, batch_size=int(gpu_cnt*128), num_workers=0, shuffle=False, collate_fn=eval_dataset.collate)

    device = "cuda"
    model = FIBERTransformerSS(_config)
    model.setup("test")
    model.eval()
    model = torch.nn.DataParallel(model).to(device)


    # 3.3. FORWARD PROCESS

    def infer_batch(batch):

        with torch.no_grad():
            scores = {}
            infer = model(batch)
            itm_logits = model.module.itm_score(infer["cls_feats"])
            scores['itm_score_logits'] = itm_logits[:, 1]
            score = torch.nn.functional.softmax(itm_logits)[:, 1]
            scores['itm_score'] = score
        return scores


    eval_root_dir = _config["eval_root"] if _config["eval_root"] is not None else os.path.join('result', _config["exp_name"])
    if not os.path.exists(os.path.join(eval_root_dir, eval_results_path)):
        os.makedirs(os.path.join(eval_root_dir, eval_results_path))


    if _config["eval_data"] == "eqben":
        fiber_score_all = []
        for idx, (image, encode, caption) in enumerate(tqdm(eval_dataloader)):
            image, encode = image.to(device), encode.to(device)
            batch_c_i = {"text": caption, "image": [image], "text_ids": encode["input_ids"], "text_labels": encode["input_ids"], "text_masks": encode["attention_mask"]}
            fiber_score = infer_batch(batch_c_i)
            fiber_score_all.append(fiber_score["itm_score"].cpu())

        fiber_score_all = torch.cat(fiber_score_all, dim=0)
        np.save('{}/{}/eqben_scores.npy'.format(_config["eval_root"], eval_results_path), fiber_score_all)

    else: # winoground / valse
        fiber_score_c0_i0_all_itm, fiber_score_c1_i0_all_itm, fiber_score_c0_i1_all_itm, fiber_score_c1_i1_all_itm = [], [], [], []
        for idx, (image0, image1, encode0, encode1, caption0, caption1) in enumerate(tqdm(eval_dataloader)):

            image0, image1, encode0, encode1 = image0.to(device), image1.to(device), encode0.to(device), encode1.to(device)
            batch_c0_i0 = {"text":caption0, "image":[image0], "text_ids":encode0["input_ids"], "text_labels":encode0["input_ids"], "text_masks":encode0["attention_mask"]}
            batch_c1_i0 = {"text":caption1, "image":[image0], "text_ids":encode1["input_ids"], "text_labels":encode1["input_ids"], "text_masks":encode1["attention_mask"]}
            batch_c0_i1 = {"text":caption0, "image":[image1], "text_ids":encode0["input_ids"], "text_labels":encode0["input_ids"], "text_masks":encode0["attention_mask"]}
            batch_c1_i1 = {"text":caption1, "image":[image1], "text_ids":encode1["input_ids"], "text_labels":encode1["input_ids"], "text_masks":encode1["attention_mask"]}

            fiber_score_c0_i0_dict = infer_batch(batch_c0_i0)
            fiber_score_c1_i0_dict = infer_batch(batch_c1_i0)
            fiber_score_c0_i1_dict = infer_batch(batch_c0_i1)
            fiber_score_c1_i1_dict = infer_batch(batch_c1_i1)

            fiber_score_c0_i0_all_itm.append(fiber_score_c0_i0_dict["itm_score"].cpu())
            fiber_score_c1_i0_all_itm.append(fiber_score_c1_i0_dict["itm_score"].cpu())
            fiber_score_c0_i1_all_itm.append(fiber_score_c0_i1_dict["itm_score"].cpu())
            fiber_score_c1_i1_all_itm.append(fiber_score_c1_i1_dict["itm_score"].cpu())



        fiber_score_c0_i0_all_itm = torch.cat(fiber_score_c0_i0_all_itm, dim=0)
        fiber_score_c1_i0_all_itm = torch.cat(fiber_score_c1_i0_all_itm, dim=0)
        fiber_score_c0_i1_all_itm = torch.cat(fiber_score_c0_i1_all_itm, dim=0)
        fiber_score_c1_i1_all_itm = torch.cat(fiber_score_c1_i1_all_itm, dim=0)
        eval_fiber_scores_itm = {'raw_info':eval_dataset.sample_pair, "c0_i0": fiber_score_c0_i0_all_itm, "c0_i1": fiber_score_c0_i1_all_itm, "c1_i0": fiber_score_c1_i0_all_itm, "c1_i1": fiber_score_c1_i1_all_itm, "model_path": _config["load_path"]}



################ 4. CALCULATE/SAVE THE PERFORMANCE #####################

    if 'valse' in _config["eval_data"]:
        score_valse_acc, score_valse_min, score_valse_pair = eqben_acc.cal_valse_score(eval_fiber_scores_itm)
        logger_txt.info("============ ITM Score =============")
        logger_txt.info("valse score pair: {}".format(score_valse_pair))
        logger_txt.info("valse score min(pc,pf): {}".format(score_valse_min))
        logger_txt.info("valse score acc: {}".format(score_valse_acc))

    else:
        text_score_itm, image_score_itm, group_score_itm = eqben_acc.cal_wino_score(eval_fiber_scores_itm)
        eval_fiber_scores_itm["scores"] = {"text_score":text_score_itm, "image_score":image_score_itm, "group_score":group_score_itm}
        logger_txt.info("============ ITM Score =============")
        logger_txt.info("text score: {}".format(text_score_itm))
        logger_txt.info("image score: {}".format(image_score_itm))
        logger_txt.info("group score: {}".format(group_score_itm))





# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved.
import logging
import os
from logging import StreamHandler, Handler, getLevelName


# this class is a copy of logging.FileHandler except we end self.close()
# at the end of each emit. While closing file and reopening file after each
# write is not efficient, it allows us to see partial logs when writing to
# fused Azure blobs, which is very convenient
class FileHandler(StreamHandler):
    """
    A handler class which writes formatted logging records to disk files.
    """
    def __init__(self, filename, mode='a', encoding=None, delay=False):
        """
        Open the specified file and use it as the stream for logging.
        """
        # Issue #27493: add support for Path objects to be passed in
        filename = os.fspath(filename)
        #keep the absolute path, otherwise derived classes which use this
        #may come a cropper when the current directory changes
        self.baseFilename = os.path.abspath(filename)
        self.mode = mode
        self.encoding = encoding
        self.delay = delay
        if delay:
            #We don't open the stream, but we still need to call the
            #Handler constructor to set level, formatter, lock etc.
            Handler.__init__(self)
            self.stream = None
        else:
            StreamHandler.__init__(self, self._open())

    def close(self):
        """
        Closes the stream.
        """
        self.acquire()
        try:
            try:
                if self.stream:
                    try:
                        self.flush()
                    finally:
                        stream = self.stream
                        self.stream = None
                        if hasattr(stream, "close"):
                            stream.close()
            finally:
                # Issue #19523: call unconditionally to
                # prevent a handler leak when delay is set
                StreamHandler.close(self)
        finally:
            self.release()

    def _open(self):
        """
        Open the current base file with the (original) mode and encoding.
        Return the resulting stream.
        """
        return open(self.baseFilename, self.mode, encoding=self.encoding)

    def emit(self, record):
        """
        Emit a record.
        If the stream was not opened because 'delay' was specified in the
        constructor, open it before calling the superclass's emit.
        """
        if self.stream is None:
            self.stream = self._open()
        StreamHandler.emit(self, record)
        self.close()

    def __repr__(self):
        level = getLevelName(self.level)
        return '<%s %s (%s)>' % (self.__class__.__name__, self.baseFilename, level)


_LOG_FMT = '%(asctime)s - %(levelname)s - %(name)s -   %(message)s'
_DATE_FMT = '%m/%d/%Y %H:%M:%S'
logging.basicConfig(format=_LOG_FMT, datefmt=_DATE_FMT, level=logging.INFO)
LOGGER = logging.getLogger('__main__')  # this is the global logger
logging.getLogger('matplotlib.font_manager').disabled = True  # cclin

def add_log_to_file(log_path):
    fh = FileHandler(log_path)
    formatter = logging.Formatter(_LOG_FMT, datefmt=_DATE_FMT)
    fh.setFormatter(formatter)
    LOGGER.addHandler(fh)



class RunningMeter(object):
    """ running meteor of a scalar value
        (useful for monitoring training loss)
    """
    def __init__(self, name, val=None, smooth=0.99):
        self._name = name
        self._sm = smooth
        self._val = val

    def __call__(self, value):
        self._val = (value if self._val is None
                     else value*(1-self._sm) + self._val*self._sm)

    def __str__(self):
        return f'{self._name}: {self._val:.4f}'

    @property
    def val(self):
        return self._val

    @property
    def name(self):
        return self._name


import json
import os
import torch
from torch.utils.data import Dataset


############# EqBen ############
class EqBenALL(Dataset):
    def __init__(self, img_root, ann_root, config, customized_data_toolkit):
        super(EqBenALL, self).__init__()
        self.config = config
        self.img_root = img_root
        self.global_ann = json.load(open(ann_root, 'r'))
        self.customized_data_toolkit = customized_data_toolkit

    def __getitem__(self, index):
        database = self.global_ann[index]['image'].split('/')[0]
        img_path = os.path.join(self.img_root, self.global_ann[index]['image'])
        caption = self.global_ann[index]['caption']
        if database == 'eqbenyoucook2':
            image = self.customized_data_toolkit.process_img_npy(img_path)
        else:
            image = self.customized_data_toolkit.process_img_pixel(img_path)
        return image, caption

    def __len__(self):
        return len(self.global_ann)

    def collate(self, batch):
        if hasattr(self.customized_data_toolkit, 'collate'): # use customized collact
            return self.customized_data_toolkit.collate_eqben(batch)
        else: # use the default pytorch collact
            return torch.utils.data.default_collate(batch)




class EqBenAG(Dataset):
    def __init__(self, img_root, ann_root, config, customized_data_toolkit):
        super(EqBenAG, self).__init__()
        self.config = config
        self.sample_pair = []
        self.global_ann = json.load(open(ann_root, 'r'))
        self.customized_data_toolkit = customized_data_toolkit
        # import pdb; pdb.set_trace()
        for process_idx, (video_dir, video_dir_values) in enumerate(self.global_ann.items()):
            print('Loading data {}/{} '.format(process_idx, len(self.global_ann)), end='\r')
            for subject_dir, subject_dir_values in video_dir_values.items():
                for frame_info in subject_dir_values:
                    image0_path, image1_path = os.path.join(img_root, frame_info['image0']), os.path.join(img_root, frame_info['image1'])
                    caption0, caption1 = frame_info['caption0'][0], frame_info['caption1'][0] # only use the first caption in the caption list
                    added_sample = {'subject':subject_dir, 'image0':image0_path, 'caption0':caption0, 'image1':image1_path, 'caption1':caption1}
                    self.sample_pair.append(added_sample)

    def __getitem__(self, index):
        image0, caption0, image1, caption1 = self.sample_pair[index]['image0'], self.sample_pair[index]['caption0'], \
                                                      self.sample_pair[index]['image1'], self.sample_pair[index]['caption1']
        image0, image1 = self.customized_data_toolkit.process_img_pixel(image0), self.customized_data_toolkit.process_img_pixel(image1)
        return image0, image1, caption0, caption1

    def __len__(self):
        return len(self.sample_pair)

    def collate(self, batch):
        if hasattr(self.customized_data_toolkit, 'collate'): # use customized collact
            return self.customized_data_toolkit.collate(batch)
        else: # use the default pytorch collact
            return torch.utils.data.default_collate(batch)



class EqBenYoucook2(Dataset):
    def __init__(self, img_root, ann_root, config, customized_data_toolkit):
        super(EqBenYoucook2, self).__init__()
        self.config = config
        self.sample_pair = []
        self.global_ann = json.load(open(ann_root, 'r'))
        self.customized_data_toolkit = customized_data_toolkit
        # import pdb; pdb.set_trace()
        for process_idx, (video_dir, video_dir_values) in enumerate(self.global_ann.items()):
            print('Loading data {}/{} '.format(process_idx, len(self.global_ann)), end='\r')
            for frame_info in video_dir_values:
                image0_path, image1_path = os.path.join(img_root, frame_info['image0']), os.path.join(img_root, frame_info['image1'])
                caption0, caption1 = frame_info["caption0"], frame_info["caption1"]
                added_sample = {'image0':image0_path, 'caption0':caption0, 'image1':image1_path, 'caption1':caption1}
                self.sample_pair.append(added_sample)

    def __getitem__(self, index):
        image0, caption0, image1, caption1 = self.sample_pair[index]['image0'], self.sample_pair[index]['caption0'], \
                                                      self.sample_pair[index]['image1'], self.sample_pair[index]['caption1']
        image0, image1 = self.customized_data_toolkit.process_img_npy(image0), self.customized_data_toolkit.process_img_npy(image1)
        return image0, image1, caption0, caption1

    def __len__(self):
        return len(self.sample_pair)

    def collate(self, batch):
        if hasattr(self.customized_data_toolkit, 'collate'): # use customized collact
            return self.customized_data_toolkit.collate(batch)
        else: # use the default pytorch collact
            return torch.utils.data.default_collate(batch)



class EqBenGEBC(Dataset):
    def __init__(self, img_root, ann_root, config, customized_data_toolkit):
        super(EqBenGEBC, self).__init__()
        self.config = config
        self.sample_pair = []
        self.global_ann = json.load(open(ann_root, 'r'))
        self.customized_data_toolkit = customized_data_toolkit
        # import pdb; pdb.set_trace()
        for process_idx, (video_dir, video_dir_values) in enumerate(self.global_ann.items()):
            print('Loading data {}/{} '.format(process_idx, len(self.global_ann)), end='\r')
            for frame_info in video_dir_values:
                image0_path, image1_path = os.path.join(img_root, frame_info['image0']), os.path.join(img_root, frame_info['image1'])
                caption0, caption1 = frame_info["caption0"], frame_info["caption1"]
                added_sample = {'image0':image0_path, 'caption0':caption0, 'image1':image1_path, 'caption1':caption1}
                self.sample_pair.append(added_sample)

    def __getitem__(self, index):
        image0, caption0, image1, caption1 = self.sample_pair[index]['image0'], self.sample_pair[index]['caption0'], \
                                                      self.sample_pair[index]['image1'], self.sample_pair[index]['caption1']
        image0, image1 = self.customized_data_toolkit.process_img_pixel(image0), self.customized_data_toolkit.process_img_pixel(image1)
        return image0, image1, caption0, caption1

    def __len__(self):
        return len(self.sample_pair)

    def collate(self, batch):
        if hasattr(self.customized_data_toolkit, 'collate'): # use customized collact
            return self.customized_data_toolkit.collate(batch)
        else: # use the default pytorch collact
            return torch.utils.data.default_collate(batch)




class EqBenKubric(Dataset):
    def __init__(self, img_root, ann_root, config, customized_data_toolkit):
        super(EqBenKubric, self).__init__()
        self.config = config
        self.sample_pair = []
        self.global_ann = json.load(open(ann_root, 'r'))
        self.customized_data_toolkit = customized_data_toolkit
        # import pdb; pdb.set_trace()
        for process_idx, frame_info in enumerate(self.global_ann):
            print('Loading data {}/{} '.format(process_idx, len(self.global_ann)), end='\r')
            image0_path, image1_path = os.path.join(img_root, frame_info['image0']), os.path.join(img_root, frame_info['image1'])
            caption0, caption1 = frame_info['caption0'], frame_info['caption1']
            added_sample = {'image0':image0_path, 'caption0':caption0, 'image1':image1_path, 'caption1':caption1}
            self.sample_pair.append(added_sample)

    def __getitem__(self, index):
        image0, caption0, image1, caption1 = self.sample_pair[index]['image0'], self.sample_pair[index]['caption0'], \
                                                      self.sample_pair[index]['image1'], self.sample_pair[index]['caption1']
        image0, image1 = self.customized_data_toolkit.process_img_pixel(image0), self.customized_data_toolkit.process_img_pixel(image1)
        return image0, image1, caption0, caption1

    def __len__(self):
        return len(self.sample_pair)

    def collate(self, batch):
        if hasattr(self.customized_data_toolkit, 'collate'): # use customized collact
            return self.customized_data_toolkit.collate(batch)
        else: # use the default pytorch collact
            return torch.utils.data.default_collate(batch)




class EqBenSD(Dataset):
    def __init__(self, img_root, ann_root, config, customized_data_toolkit):
        super(EqBenSD, self).__init__()
        self.config = config
        self.sample_pair = []
        self.global_ann = json.load(open(ann_root, 'r'))
        self.customized_data_toolkit = customized_data_toolkit

        for process_idx, frame_info in enumerate(self.global_ann):
            print('Loading data {}/{} '.format(process_idx, len(self.global_ann)), end='\r')
            image0_path, image1_path = os.path.join(img_root, frame_info['image0']), os.path.join(img_root, frame_info['image1'])
            caption0, caption1 = frame_info['caption0'], frame_info['caption1']
            added_sample = {'image0':image0_path, 'caption0':caption0, 'image1':image1_path, 'caption1':caption1}
            self.sample_pair.append(added_sample)

    def __getitem__(self, index):
        image0, caption0, image1, caption1 = self.sample_pair[index]['image0'], self.sample_pair[index]['caption0'], \
                                                      self.sample_pair[index]['image1'], self.sample_pair[index]['caption1']
        image0, image1 = self.customized_data_toolkit.process_img_pixel(image0), self.customized_data_toolkit.process_img_pixel(image1)
        return image0, image1, caption0, caption1

    def __len__(self):
        return len(self.sample_pair)

    def collate(self, batch):
        if hasattr(self.customized_data_toolkit, 'collate'): # use customized collact
            return self.customized_data_toolkit.collate(batch)
        else: # use the default pytorch collact
            return torch.utils.data.default_collate(batch)



####### Conventional Benchmarks #######

class Winoground(Dataset):
    def __init__(self, img_root, ann_root, config, customized_data_toolkit):
        super(Winoground, self).__init__()
        self.config = config
        self.sample_pair = []
        self.global_ann = [json.loads(example_json) for example_json in open(ann_root).readlines()]
        self.customized_data_toolkit = customized_data_toolkit
        # import pdb; pdb.set_trace()
        for process_idx, frame_info in enumerate(self.global_ann):
            print('Loading data {}/{} '.format(process_idx, len(self.global_ann)), end='\r')
            image0_path, image1_path = os.path.join(img_root, frame_info['image_0']+'.png'), os.path.join(img_root, frame_info['image_1']+'.png')
            caption0, caption1 = frame_info['caption_0'], frame_info['caption_1']
            added_sample = {'image0':image0_path, 'caption0':caption0, 'image1':image1_path, 'caption1':caption1}
            self.sample_pair.append(added_sample)

    def __getitem__(self, index):
        image0, caption0, image1, caption1 = self.sample_pair[index]['image0'], self.sample_pair[index]['caption0'], \
                                                      self.sample_pair[index]['image1'], self.sample_pair[index]['caption1']
        image0, image1 = self.customized_data_toolkit.process_img_pixel(image0), self.customized_data_toolkit.process_img_pixel(image1)
        return image0, image1, caption0, caption1

    def __len__(self):
        return len(self.sample_pair)

    def collate(self, batch):
        if hasattr(self.customized_data_toolkit, 'collate'): # use customized collact
            return self.customized_data_toolkit.collate(batch)
        else: # use the default pytorch collact
            return torch.utils.data.default_collate(batch)




class VALSE(Dataset):
    def __init__(self, img_root, ann_root, config, customized_data_toolkit):
        super(VALSE, self).__init__()
        self.config = config
        self.sample_pair = []
        self.global_ann = json.load(open(ann_root, 'r'))
        self.customized_data_toolkit = customized_data_toolkit
        # import pdb; pdb.set_trace()
        for process_idx, frame_info in enumerate(self.global_ann):
            print('Loading data {}/{} '.format(process_idx, len(self.global_ann)), end='\r')
            image0_path, image1_path = os.path.join(img_root, frame_info['image_0']), os.path.join(img_root, frame_info['image_1'])
            caption0, caption1 = frame_info['caption_0'], frame_info['caption_1']
            added_sample = {'image0':image0_path, 'caption0':caption0, 'image1':image1_path, 'caption1':caption1}
            self.sample_pair.append(added_sample)

    def __getitem__(self, index):
        image0, caption0, image1, caption1 = self.sample_pair[index]['image0'], self.sample_pair[index]['caption0'], \
                                                      self.sample_pair[index]['image1'], self.sample_pair[index]['caption1']
        image0, image1 = self.customized_data_toolkit.process_img_pixel(image0), self.customized_data_toolkit.process_img_pixel(image1)
        return image0, image1, caption0, caption1

    def __len__(self):
        return len(self.sample_pair)

    def collate(self, batch):
        if hasattr(self.customized_data_toolkit, 'collate'): # use customized collact
            return self.customized_data_toolkit.collate(batch)
        else: # use the default pytorch collact
            return torch.utils.data.default_collate(batch)



import torch
import copy
import os
from tqdm import tqdm
from torch.utils.data import Dataset
from PIL import Image
import numpy as np
import eqben.eqben_utils as eqben_utils
import eqben.eqben_acc as eqben_acc


##################### 0. IMPORT USER-MODEL PACKAGE ###########################
# In this template, we use "USER_MODEL" to represent the VLP model which would be evaluated.
# Please check other files within the example folder for the specific VLP model.




##################### 1. DATA SET PATH ###########################
# Please identify the path of "img_root" and "ann_root" in the server.

DATASET_INFO = {
    "eqben": {"func": eqben_utils.EqBenALL,
              "img_root": '',
              "ann_root": ''},
    'winoground': {"func": eqben_utils.Winoground,
                   'img_root':'',
                   'ann_root':''},
    'valse': {"func": eqben_utils.VALSE,
              'img_root':'',
              'ann_root':''}
}



################ 2. CUSTOMIZED DATA TOOL #####################
# In USER_MODEL, how will the data (image/caption) be pre-processed?
# How will the data be collated in the batch?


class customized_data_toolkit():
    def __init__(self):
        pass

    def process_img_pixel(self, image):
        # Define the model-specifc data pre-process (e.g., augmentation) for image pixel.
        pass

    def process_img_npy(self, image):
        # Define the model-specifc data pre-process (e.g., augmentation) for image numpy file (Youcook2).
        pass

    def collate(self, batch):
        # Define the model-specific data collate method, will be forwarded to "collate_fn"
        pass




################ 3. USER_MODEL INFERENCE PROCESS #####################
# For USER_MODEL, how to forward the data (image/caption) and obtain the image-text simiarity during the inference stage.
config = {"eval_data": "eqbenag"}


# 3.1. SETUP LOGGER
eval_results_path = ''
os.makedirs(os.path.join(eval_results_path), exist_ok=True)
from eqben.eqben_logger import LOGGER as logger_txt, add_log_to_file
logger_path = '{}/xxx'.format(eval_results_path)
logger_txt.info("creating log at: {}".format(logger_path))
add_log_to_file(logger_path)


# 3.2. DATALOADER/USER_MODEL SETUP
eval_dataset = DATASET_INFO[config["eval_data"]]["func"](img_root=DATASET_INFO[config["eval_data"]]["img_root"], ann_root=DATASET_INFO[config["eval_data"]]["ann_root"], config=config,
                                                          customized_data_toolkit=customized_data_toolkit()) # automatic create PyTorch dataset for the evaluated data
gpu_cnt = torch.cuda.device_count()
print('use {} GPUs; '.format(gpu_cnt))
eval_dataloader = torch.utils.data.DataLoader(eval_dataset, batch_size=int(gpu_cnt * 128), num_workers=int(gpu_cnt*4), shuffle=False, collate_fn=eval_dataset.collate)

USER_MODEL = None


# 3.3. FORWARD PROCESS
if config.eval_data == "eqben": # for eqben, please send the result file to the server
    score_all = []
    for idx, (image, caption) in enumerate(tqdm(eval_dataloader)):
        score = USER_MODEL(image, caption)
        score_all.append(score.cpu())
    score_all = torch.cat(score_all, dim=0)
    np.save('{}/eqben_scores.npy'.format(eval_results_path), score_all)


else: # for winoground/valse, directly calculate
    score_c0_i0_all, score_c1_i0_all, score_c0_i1_all, score_c1_i1_all = [], [], [], []
    for idx, (image0, image1, caption0, caption1) in enumerate(tqdm(eval_dataloader)):
        score_c0_i0, score_c1_i0, score_c0_i1, score_c1_i1 = USER_MODEL(image0, image1, caption0, caption1)
        score_c0_i0_all.append(score_c0_i0)
        score_c1_i0_all.append(score_c1_i0)
        score_c0_i1_all.append(score_c0_i1)
        score_c1_i1_all.append(score_c1_i1)
    score_c0_i0_all = torch.cat(score_c0_i0_all, dim=0)
    score_c1_i0_all = torch.cat(score_c1_i0_all, dim=0)
    score_c0_i1_all = torch.cat(score_c0_i1_all, dim=0)
    score_c1_i1_all = torch.cat(score_c1_i1_all, dim=0)
    eval_scores_dict = {'raw_info':eval_dataset.sample_pair, "c0_i0": score_c0_i0_all, "c0_i1": score_c0_i1_all, "c1_i0": score_c1_i0_all, "c1_i1": score_c1_i1_all, "model_path": config["load_path"]}



    ################ 4. CALCULATE/SAVE THE PERFORMANCE #####################

    if 'valse' in config["eval_data"]:
        score_valse_acc, score_valse_min, score_valse_pair = eqben_acc.cal_valse_score(eval_scores_dict)
        logger_txt.info("============ Score =============")
        logger_txt.info("valse score pair: {}".format(score_valse_pair))
        logger_txt.info("valse score min(pc,pf): {}".format(score_valse_min))
        logger_txt.info("valse score acc: {}".format(score_valse_acc))

    else:
        text_score_itm, image_score_itm, group_score_itm = eqben_acc.cal_wino_score(eval_scores_dict)
        eval_scores_dict["scores"] = {"text_score": text_score_itm, "image_score": image_score_itm, "group_score": group_score_itm}
        logger_txt.info("============ Score =============")
        logger_txt.info("text score: {}".format(text_score_itm))
        logger_txt.info("image score: {}".format(image_score_itm))
        logger_txt.info("group score: {}".format(group_score_itm))





import torch


# calculate the score proposed in wionground
def cal_wino_score(result):
    def text_correct(result):
        return torch.logical_and(result["c0_i0"] > result["c1_i0"], result["c1_i1"] > result["c0_i1"])

    def image_correct(result):
        return torch.logical_and(result["c0_i0"] > result["c0_i1"], result["c1_i1"] > result["c1_i0"])

    def group_correct(result):
        return torch.logical_and(image_correct(result), text_correct(result))

    def cal_score(list_correct):
        correct_cnt = list_correct.sum()
        denominator = len(list_correct)
        return correct_cnt / denominator

    return cal_score(text_correct(result)), cal_score(image_correct(result)), cal_score(group_correct(result))


# calculate the score proposed in VALSE
def cal_valse_score(result):
    def cal_valse_score_pair(result):
        return (result["c0_i0"] > result["c1_i0"])

    def cal_valse_score_pc(result):
        return (result["c0_i0"] > 0.5)

    def cal_valse_score_pf(result):
        return (result["c1_i0"] < 0.5)

    def cal_valse_score_acc(result):
        true_cnt = (result["c0_i0"] > 0.5).sum() + (result["c1_i0"] < 0.5).sum()
        return true_cnt / (len(result["c0_i0"]) * 2)

    def cal_score(list_correct):
        correct_cnt = list_correct.sum()
        denominator = len(list_correct)
        return correct_cnt / denominator

    return cal_valse_score_acc(result), min(cal_score(cal_valse_score_pc(result)), cal_score(cal_valse_score_pf(result))), cal_score(cal_valse_score_pair(result))

